# Copyright (c) KylinSoft Co., Ltd. [2024].All rights reserved. 

#!/bin/bash


unset LANG
os_full_name=`cat /etc/os-release | head -1 | awk -F "release" '{print $1}'`
if [[ `echo $os_full_name | grep -i "red hat"` ]]
then
	os_name="rhel"
elif [[ `echo $os_full_name | grep -i "KYLIN"` ]]
then
	os_name="kylin"
elif [[ `echo $os_full_name | grep -i "Ubuntu"` ]]
then
	os_name="Ubuntu"
elif [[ `echo $os_full_name | grep -i "Debian"` ]]
then
	os_name="Debian"
fi

if [[ $os_name = "rhel" ]]
then
	os_version=`cat /etc/issue | head -1 | awk -F "release" '{print $2}'`
elif [[ $os_name = "kylin" ]]
then
	os_version=`cat /etc/kylin-release | head -1 | awk -F "release" '{print $2}'`
elif [[ $os_name = "Ubuntu" ]]
then
	os_version=`cat /etc/issue | head -1 | awk '{print $2}'`
elif [[ $os_name = "Debian" ]]
then
	os_version=`cat /etc/issue | head -1 | awk '{print $3}'`
fi

function get_value_with_key() {
    local line="$1"
    local key="$2"
    local value="";

    if [ -z "$line" ] || [ -z $key ]
    then
	    echo $value;
	    return;
    fi

    value=$(echo $line | grep -o "[^ ]\+" | grep "^$key=" | cut -f2 -d '=')
    echo $value
}

function fun_deny_unlock {
	if [ ! -e "/etc/pam.d/$1" ]; then
		return 0;
	fi

	local match=$(grep -E "^[[:space:]]*auth[[:space:]]+(required|requisite)[[:space:]]+(pam_faillock.so|pam_tally2.so)[[:space:]]+.*deny=[0-9]+.*unlock_time=[0-9]+" /etc/pam.d/$1)

	if [ -n "$match" ]
	then
		while read -r line
		do
			#echo "处理行：$line"
			local pam_module=$(find /usr -name $(echo $line | awk '{print $3}') 2> /dev/null)
			if [ -z "$pam_module" ]
			then
				continue
			fi

			local deny=$(get_value_with_key "$line" "deny")
			local unlock_time=$(get_value_with_key "$line" "unlock_time")

			#echo "deny:$deny"
			#echo "unlock_time:$unlock_time"

			if [ $deny -le 5 ]&&[ $unlock_time -ge 120 ]
			then
				return 1;
			else
				return 0;
			fi
		done <<< "$match"
	fi

	return 0;
}

function fun_1 {
	local result=0

	for file in login sshd gdm system-auth
	do
		fun_deny_unlock "$file"
		result=$?

		if [ $result -eq 1 ]; then
			echo yes
			return
		fi
	done

	echo no
}
code[1]=$(declare -f fun_deny_unlock fun_1)


function fun_2 {
	#find / -type f -perm /u+s,g+s 2> /dev/null | ls -l
	for  PART  in `grep -v ^# /etc/fstab | awk '($6 != "0") {print $2 }'`
        do
            find $PART -type f \( -perm -04000 -o -perm -02000 \)  -xdev -exec ls -lg {} \; 2>>/dev/null >/dev/null
        done
	#ls -l $search
	echo yes
}
code[2]=$(declare -f fun_2)


function fun_3 {
	timeout=0

	oldIFS=$IFS
	IFS=$'\n'
	for line in `cat /etc/profile`
	do
        	if [ -n "`echo $line | grep '^[[:space:]]*TMOUT=[0-9]*[[:space:]]*'`" ]
        	then
                	timeout=`echo $line | cut -f2 -d '='`
        	fi
	done
	if [ $timeout -eq 300 ]
	then
		echo yes
	else
		echo no
	fi

	#echo timeout=$timeout
	IFS=$oldIFS

	unset timeout oldIFS line
}
code[3]=$(declare -f fun_3)


function fun_4 {
	ssh_PermitRootLogin=1
	protocol_num=0
	if [ -s /etc/ssh/sshd_config ]
	then
        	if [[ `cat /etc/ssh/sshd_config | grep '[[:space:]]*PermitRootLogin[[:space:]]*no[[:space:]]*'` ]]
        	then
                	ssh_PermitRootLogin=0
        	fi

		if [[ `cat /etc/ssh/sshd_config | grep '[[:space:]]*Protocol[[:space:]]*2[[:space:]]*'` ]]
		then
			protocol_num=2
		fi
	fi

	if [ $ssh_PermitRootLogin -eq 0 ]
	then
		echo yes
	else
		echo no
	fi

	unset ssh_PermitRootLogin protocol_num
}
code[4]=$(declare -f fun_4)


function fun_5 {
	ssh_status=`ps -ef | grep sshd | grep -v grep`
	if [[ -z "$ssh_status" ]]
	then
        	ssh_running=0
	else
        	ssh_running=1
	fi
	telnet_status=`ps -ef | grep telnet | grep -v grep`
	if [[ -z "$telnet_status" ]]
	then
        	telnet_running=1
	else
        	telnet_running=0
	fi

	if [ $ssh_running -eq 1 ] && [ $telnet_running -eq 1 ]
	then
		echo yes
	else
		echo no
	fi
	unset ssh_running ssh_status 
}
code[5]=$(declare -f fun_5)


function fun_6 {
	find / -maxdepth 3 -name .netrc 2>/dev/null
	find / -maxdepth 3 -name .rhosts 2>/dev/null
	find / -maxdepth 3 -name hosts.equiv 2>/dev/null
	totalNum_netrc=`find / -maxdepth 3 -name .netrc 2>/dev/null|wc -l`
	totalNum_rhosts=`find / -maxdepth 3 -name .rhosts 2>/dev/null|wc -l`
	totalNum_hosts_equiv=`find / -maxdepth 3 -name hosts.equiv 2>/dev/null|wc -l`
	
	#echo totalNum_netrc=$totalNum_netrc totalNum_rhosts=$totalNum_rhosts totalNum_hosts.equiv=$totalNum_hosts_equiv
	if [ $totalNum_netrc -eq 0 ] && [ $totalNum_rhosts -eq 0 ] && [ $totalNum_hosts_equiv -eq 0 ]
	then	
		echo yes
	else	
		echo no
	fi
	unset totalNum_netrc totalNum_rhosts totalNum_hosts_equiv
}
code[6]=$(declare -f fun_6)


function fun_7 {
	umask1=`cat /etc/bashrc | grep "^[[:space:]]*umask" | grep -v ^# | awk '{print $2}'`
	if [ -n "`echo $umask1 | grep '027'`" ]
	then	
		echo yes
	else
		echo no
	fi
	unset umask1
}
code[7]=$(declare -f fun_7)


function fun_8 {
	if [[ -f /etc/passwd ]]
	then
		file_passwd=`ls -l /etc/passwd | awk '{print $1}'`
		passwd_num=`stat -c "%a" /etc/passwd`
	else
		passwd_num=777
	fi

	if [ -f /etc/shadow ]
	then
		shadow_num=`stat -c "%a" /etc/shadow`
	else
		shadow_num=777
	fi

	if [ -f /etc/group ]
	then
		group_num=`stat -c "%a" /etc/group`
	else
		group_num=777
	fi

	if [ $passwd_num -le 644 ]&&[ $shadow_num -le 600 ]&&[ $group_num -le 644 ]
	then
		echo yes
	else
		echo no
	fi
	unset group_num shadow_num group_num
}
code[8]=$(declare -f fun_8)


function fun_9 {
	passwd_max_days=`cat /etc/login.defs | egrep '^[[:space:]]*PASS_MAX_DAYS[[:space:]]*[0-9][0-9]*[[:space:]]*$' | awk '{print $2}'`
	passwd_min_days=`cat /etc/login.defs | egrep '^[[:space:]]*PASS_MIN_DAYS[[:space:]]*[0-9][0-9]*[[:space:]]*$' | awk '{print $2}'`
	passwd_warn_days=`cat /etc/login.defs | egrep '^[[:space:]]*PASS_WARN_AGE[[:space:]]*[0-9][0-9]*[[:space:]]*$' | awk '{print $2}'`

	#echo passwd_max_days=$passwd_max_days
	#echo passwd_min_days=$passwd_min_days
	#echo passwd_warn_days=$passwd_warn_days

	if [[ $passwd_max_days -le 90 ]]
	then
		echo yes
	else
		echo no
	fi

	unset passwd_max_days passwd_min_days passwd_warn_days
}
code[9]=$(declare -f fun_9)


function fun_10 {
	count=0
	oldIFS=$IFS
	IFS=$'\n'

	for line in $(cat /etc/passwd)
	do
        	uid=`echo $line | cut -d : -f3`
        	if [ $uid -eq 0 ]
        	then
                	count=$(expr $count + 1)
        	fi
	done

	#echo $count

	if [ $count -le 1 ]
	then	
		echo yes
	else	
		echo no
	fi
	
	IFS=$oldIFS
	unset count line uid oldIFS
}
code[10]=$(declare -f fun_10)

function fun_11 {
	if [ -f "/etc/security/pwquality.conf" ];then
		minlen=$(grep -E '^\s*minlen\s*=' /etc/security/pwquality.conf | awk -F '=' '{print$2}' |awk '{print $1}')
		dcredit=$(grep -E '^\s*dcredit\s*=' /etc/security/pwquality.conf | awk -F '=' '{print$2}' |awk '{print $1}')
		lcredit=$(grep -E '^\s*lcredit\s*=' /etc/security/pwquality.conf | awk -F '=' '{print$2}' |awk '{print $1}')
		ucredit=$(grep -E '^\s*ucredit\s*=' /etc/security/pwquality.conf | awk -F '=' '{print$2}' |awk '{print $1}')
		ocredit=$(grep -E '^\s*ocredit\s*=' /etc/security/pwquality.conf | awk -F '=' '{print$2}' |awk '{print $1}')
		minclass=$(grep -E '^\s*minclass\s*=' /etc/security/pwquality.conf | awk -F '=' '{print$2}' |awk '{print $1}')

		if [ "$minlen" = "8" ] && [[ "$dcredit" = "-1" ]] && [[ "$lcredit" = "-1" ]] && [[ "$ucredit" = "-1" ]] && [[ "$ocredit" = "-1" ]] & [ "$minclass" = "3" ]
		then
			echo yes
		else
			echo no
		fi

		unset minlen
		unset ucredit
		unset lcredit
		unset dcredit
		unset ocredit
		unset minclass
	else
		echo no
	fi
}
code[11]=$(declare -f fun_11)

# function fun_11 {
# 	input=`cat /etc/pam.d/system-auth | gawk '/^[[:space:]]*password[[:space:]]*requisite[[:space:]]*pam_cracklib\.so[[:space:]]*/{print $0}'`

# 	# 使用 awk 进行字符串分割和提取
# 	fields=$(awk -F ' ' '{print $4, $5, $6, $7, $8, $9, $10}' <<< "$input")

# 	# 分割字段
# 	read -r -a array <<< "$fields"

# 	# 遍历并解析字段
# 	for ((i = 0; i < ${#array[@]}; i++))
# 	do
# 	    str=${array[$i]}
# 	    key=$(awk -F '=' '{print $1}' <<< "$str")
# 	    value=$(awk -F '=' '{print $2}' <<< "$str")
	
# 	    # 将字段值赋给与key同名的变量
# 	    eval "$key=\"$value\""
# 	done

# 	#echo retry=$retry
# 	#echo difok=$difok
# 	#echo minlen=$minlen
# 	#echo ucredit=$ucredit
# 	#echo lcredit=$lcredit
# 	#echo dcredit=$dcredit
# 	#echo ocredit=$ocredit
# 	if [ "$retry"="3" ] && [[ "$dcredit" = "-1" ]] && [[ "$lcredit" = "-1" ]] && [[ "$ucredit" = "-1" ]] && [[ "$ocredit" = "-1" ]]
# 	then
# 		echo yes
# 	else
# 		echo no
# 	fi

# 	unset retry
# 	unset difok
# 	unset minlen
# 	unset ucredit
# 	unset lcredit
# 	unset dcredit
# 	unset ocredit
# }
# code[11]=$(declare -f fun_11)


function fun_12 {
	if [ `echo $PATH | grep '\.\/'` ]
	then
        	safty=no
	else
        	safty=yes
	fi

#	echo safty=$safty
	echo $safty
	unset safty
}
code[12]=$(declare -f fun_12)


function fun_13 {
        oldIFS=$IFS
        IFS=$'\n'

	if [ -s /etc/rsyslog.conf ]; then
		num=$(netstat -lnptu | grep rsyslog | wc -l)
		if [ $num -gt 0 ]; then
			echo yes
		else
			echo no
		fi
	fi

	IFS=$oldIFS
	unset misc
}
code[13]=$(declare -f fun_13)


function fun_14 {
	ssh_status=`ps -ef | grep sshd | grep -v grep`
	if [[ -z "$ssh_status" ]]
	then
		echo no
	else
		echo yes
	fi
	unset ssh_status
}
code[14]=$(declare -f fun_14)


function fun_15(){
        declare -i flag=0
        string=""

        for user in `cat /etc/passwd|grep "/sbin/nologin"|awk -F ':' '{print $1}'`
        do
                if [ `cat /etc/shadow |grep "$user"|awk -v str="$user" -F ':' '{if($1==str) print $1}'|grep -v "#" 1>/dev/null 2>/dev/null;echo $?` -eq 0 ];then
                        if [ `passwd -S $user |awk '{if($2 !="LK" && $2 != "L")print $1}'|grep -v "#" 1>/dev/null 2>/dev/null;echo $?` -eq 0 ];then
                                flag=1
                        fi
                fi

        done

        if [ $flag -eq 1 ];then
		echo no
	else
		echo yes
        fi

	unset flag
}
code[15]=$(declare -f fun_15)


function fun_16 {
	if [[ -f /etc/passwd ]]; then
		passwd_num=`stat -c "%a" /etc/passwd`
	else
        	passwd_num=777
	fi

	if [ -f /etc/shadow ]; then
		shadow_num=`stat -c "%a" /etc/shadow`
	else
		shadow_num=777
	fi

	if [ -f /etc/group ]; then
		group_num=`stat -c "%a" /etc/group`
	else
        	group_num=777
	fi

	if [ -f /etc/services ]; then
		service_num=`stat -c "%a" /etc/services`
	else
        	service_num=777
	fi

	if [ -f /etc/inetd.conf ]; then
		inetd_num=`stat -c "%a" /etc/inetd.conf`
	else
		inetd_num=0
	fi

	if [ -d /etc/security ]; then
		security_num=`stat -c "%a" /etc/security`
	else
		security_num=0
	fi
	
	if [ $passwd_num -le 644 ]&&[ $shadow_num -le 600 ]&&[ $group_num -le 644 ]&&[ $service_num -le 644 ]&&[ $inetd_num -le 600 ]&&[ $security_num -le 755 ]
	then
		echo yes
	else
		echo no
	fi
	unset group_num shadow_num group_num service_num inetd_num security_num
}
code[16]=$(declare -f fun_16)


function fun_17 {
	flag=0
	if [ -s /etc/syslog.conf ]
	then
		oldIFS=$IFS
		IFS=$'\n'
		for line in $(cat /etc/syslog.conf)
		do
			if [ `echo $line | grep -v ^[[:space:]]*# | grep '/^[[:space:]]*\*\.err\;kern\.debug\;daemon\.notice[[:space:]]*\/var\/adm\/messages[[:space:]]*/{print $0}'` ]
			then
				#kind=`echo $line | awk '{print $1}'`
				#where=`echo $line | awk '{print $2}'`

				#IFS=';'
				#for item in $kind
				#do
				#    	echo $item=$where
				#done
				#IFS=$'\n'
				flag=1
			fi
		done

		IFS=$oldIFS
	elif [ -s /etc/rsyslog.conf ]
	then
		oldIFS=$IFS
		IFS=$'\n'

		logfile="/etc/rsyslog.conf"  # rsyslog.conf文件路径
		keyword="authpriv\.\*"  # 关键词，用于匹配安全日志规则

		# 检查是否存在包含关键词的行且没有被注释掉
		grep_result=$(grep -E "^[[:space:]]*[^#]*$keyword" "$logfile")
		
		if [[ -n "$grep_result" ]]; then
			flag=1
		else
			flag=0
		fi

		IFS=$oldIFS
	fi

	if [ $flag -eq 0 ]
	then
		echo no
	elif [ $flag -eq 1 ]
	then
		echo yes
	fi

	unset flag oldIFS line
}
code[17]=$(declare -f fun_17)


function fun_18 {
	flag=1
	oldIFS=$IFS
	IFS=$'\n'

	gid_min=`cat /etc/login.defs | grep '/^[[:space:]]*GID_MIN[[:space:]]*[[:digit:]]*[[:space:]]*' |awk '($1="GID_MIN") {print $2}'`
	gid_max=`cat /etc/login.defs | grep '/^[[:space:]]*GID_MAX[[:space:]]*[[:digit:]]*[[:space:]]*' |awk '($1="GID_MAX") {print $2}'`
	
	for line in $(cat /etc/passwd)
	do
        	gid=`echo $line | awk -F ':' '{print $4}'`
        	if [[ $gid -ge $gid_min ]] || [[ $gid -le $gid_max ]]
        	then
                	flag=1
            	else	
			flag=0
			break
        	fi
	done

	if [ $flag -eq 0 ]
	then
		echo no
	elif [ $flag -eq 1 ]
	then	
		echo yes
	fi
	unset gid condition line gid_min gid_max flag
	IFS=$oldIFS	
}
code[18]=$(declare -f fun_18)


function fun_19 {
	ip_forward=`cat /proc/sys/net/ipv4/ip_forward`
	#echo ip_forward=$ip_forward
	
	if [ $ip_forward -eq 0 ]
	then
		echo yes
	else	
		echo no
	fi
	
	unset ip_forward
}
code[19]=$(declare -f fun_19)


function fun_20 {
	oldIFS=$IFS
	IFS=$'\n'
	service="`pgrep 'nfsd$' | wc -l`"

	if [[ $service -gt 0 ]];then
		NFS=1
		if [ -s /etc/hosts.allow ]; then
			awk -F ':' '/^[[:space:]]*(all|nfs)/ { print $0; }' /etc/hosts.allow
			NFS_allow=1
		else
			NFS_allow=0
		fi

		if [ -s /etc/hosts.deny ]; then
			awk -F ':' '/^[[:space:]]*(all|nfs)/ { print $0; }' /etc/hosts.deny
			NFS_deny=1
		else
			NFS_deny=0
		fi
	else
		NFS=0
	fi

	#echo NFS=$NFS
	if [ $NFS -eq 1 ] && [ $NFS_allow -eq 1 ] && [ $NFS_deny -eq 1 ]; then
		echo yes
	elif [ $NFS -eq 0 ];then
		echo yes
	else
		echo no
	fi
	IFS=$oldIFS

	unset NFS oldIFS NFS_allow NFS_deny
}
code[20]=$(declare -f fun_20)


function fun_21 {
	cat /etc/inittab > /dev/null 2>&1
	if [ $? -eq 0 ]
	then
		line=`cat /etc/inittab | grep '/^[[:space:]]*ca\:\:ctrlaltdel\:\/sbin\/shutdown \-r \-t 4 now[[:space:]]*'`
		if [[ -n \"\$line\" ]]
		then
			echo yes
		else
			echo no
		fi
	else
		echo yes
	fi
	
	unset line
}
code[21]=$(declare -f fun_21)


function fun_22 {
	if [ ! -e /etc/hosts.allow -o ! -e /etc/hosts.deny ];then
		echo "no"
		return
	fi

	host_allow=`cat /etc/hosts.allow | grep -v "^[[:space:]]*#"`
	host_deny=`cat /etc/hosts.deny | grep -v "^[[:space:]]*#"`
	
	if [[ -n "$host_allow" ]] && [[ -n "$host_deny" ]]
	then
		echo yes
	else
		echo no
	fi
	
	unset host_allow host_deny
}
code[22]=$(declare -f fun_22)


function fun_23 {
	num=`awk -F: '$2 == ""  { print $1 }' /etc/shadow | wc -l`
	
	if [ $num -eq 0 ]
	then
		echo yes
	else
		echo no
	fi
}
code[23]=$(declare -f fun_23)


function fun_24 {
	if pgrep ntp >/dev/null && [ -s /etc/ntp.conf ] && awk '/^[[:space:]]*server/{exit 1}' /etc/ntp.conf; then
		echo "yes"
	else
		echo "yes"
	fi
}
code[24]=$(declare -f fun_24)


function fun_25 {
	net_redirect=`sysctl -n net.ipv4.conf.all.accept_redirects`
	if [ $net_redirect -eq 0 ]
	then
		echo yes
	else
		echo no
	fi
}
code[25]=$(declare -f fun_25)


function fun_26 {
	service=`ps -elf | grep snmp | grep -v grep`
	community_set=0

	if test $service
	then
		snmp=1
		#echo snmp=$snmp
		if [ -s /etc/snmpd.conf ]
		then
			#echo $snmp
			community=`cat /etc/snmpd.conf | grep -v "^[[:space:]]*#" | grep ^[[:space:]]*"rocommunity\|rwcommunity"`
			if test $community
			then
				community_set=1
			else
				community_set=0
			fi
		else
			community=0
		fi
	else
		snmp=0
	fi

	if [ $snmp -eq 1 ] && [ $community_set -eq 1 ];then
		echo yes
	elif [ $snmp -eq 0 ];then
		echo yes
	else
		echo no
	fi
	unset snmp community service community_set
}
code[26]=$(declare -f fun_26)


function fun_27 {
	type gconftool-2 >/dev/null 2>&1 || { echo no ;exit 1; }
	idle_delay=`gconftool-2 -R / | grep '/^[[:space:]]*idle_delay[[:space:]]*=[[:space:]]*[0-9]*[[:space:]]*$'`
	count=`gconftool-2 -R / | grep '/^[[:space:]]*idle_delay[[:space:]]*=[[:space:]]*[0-9]*[[:space:]]*$' | wc -l`
	count_1=0
	for element in $idle_delay
	do
		if [ "$element" -gt 0 ] 2>/dev/null ;then
	       #if grep '^[[:digit:]]*$' <<< "$element";then
	                if [[ $element -le 15 ]];then
	                        let "count_1+=1"
	                fi
	        fi
	done
	
	if [[ $count_1 -eq $count ]];then
	        echo yes
	else
	        echo no
	fi

	
	#if [[ $idle_delay -le 15 ]]
	#then
	#	echo yes
	#else
	#	echo no
	#fi
}
code[27]=$(declare -f fun_27)


function fun_28 {
	if [ -s /etc/syslog.conf ]
	then
        log=`cat /etc/syslog.conf | grep -v "^[[:space:]]*#" | grep '/^([[:space:]]*authpriv\.\*[[:space:]]*|[[:space:]]*authpriv\.info.*[[:space:]]*|[[:space:]]*\*\.\*[[:space:]]*)'`
		if [[ -n "$log" ]]
		then
			echo yes
		else
			echo no
		fi
	fi

	if [ -s /etc/rsyslog.conf ]
	then
        log=`cat /etc/rsyslog.conf | grep -v "^[[:space:]]*#" | grep -E '^([[:space:]]*authpriv\.\*[[:space:]]*|[[:space:]]*authpriv\.info.*[[:space:]]*|[[:space:]]*\*\.\*[[:space:]]*)'`
		if [[ -n "$log" ]]
		then
			echo yes
		else
			echo no
		fi
	fi
}
code[28]=$(declare -f fun_28)


function fun_29 {
	flag=0
	LOGDIR=`if [ -f /etc/syslog.conf ];then cat /etc/syslog.conf| grep -v "^[[:space:]]*#"|awk '{print $2}'|sed 's/^-//g'|grep '^\s*\/';fi`
	for file in $LOGDIR
	do
		file_perm=`ls -l $file | awk '{print $1}'`
		char1=${file_perm:0-1:1}
		if [[ $char1 = "x" ]]
                then
                        num1=1
                else
                        num1=0
                fi
		char2=${file_perm:0-2:1}
		if [[ $char2 = "w" ]]
                then
                        num2=1
                else
                        num2=0
                fi
		char3=${file_perm:0-3:1}
		if [[ $char3 = "r" ]]
                then
                        num3=1
                else
                        num3=0
                fi
		char4=${file_perm:0-4:1}
		if [[ $char4 = "x" ]]
                then
                        num4=1
                else
                        num4=0
                fi
		char5=${file_perm:0-5:1}
		if [[ $char5 = "w" ]]
                then
                        num5=1
                else
                        num5=0
                fi
		char6=${file_perm:0-6:1}
		if [[ $char1 = "r" ]]
                then
                        num6=1
                else
                        num6=0
                fi
		char7=${file_perm:0-7:1}
		if [[ $char7 = "x" ]]
                then
                        num7=1
                else
                        num7=0
                fi
		char8=${file_perm:0-8:1}
		if [[ $char8 = "w" ]]
                then
                        num8=1
                else
                        num8=0
                fi
		char9=${file_perm:0-9:1}
		if [[ $char9 = "r" ]]
                then
                        num9=1
                else
                        num9=0
                fi
		str1=$[($num3 * 4) + ($num2 * 2) + $num1]
		str2=$[($num6 * 4) + ($num5 * 2) + $num4]
		str3=$[($num9 * 4) + ($num8 * 2) + $num7]
		file_num=$[($str3 * 100) + ($str2 * 10) + $str1]
		if [ $file_num -gt 640 ]
		then
			flag=1
		else
			flag=0
		fi
	done

	if [ $flag -eq 1 ]
	then
		echo no
	else
		echo yes
	fi
}
code[29]=$(declare -f fun_29)


function fun_30 {
	fun_17
}
code[30]=$(declare -f fun_17)


function fun_31 {
	if [ -s /etc/pam.d/su ]
	then
		line=`grep -E '^[[:space:]]*auth[[:space:]]*(required|requisite)[[:space:]]*pam_wheel\.so[[:space:]]*(use_uid|group=wheel)[[:space:]]*' /etc/pam.d/su`

		if [ -n "$line" ]
		then
			su_wheel=yes
		else
			su_wheel=no
		fi
	else
		su_wheel=no
	fi

	echo $su_wheel
	unset line su_wheel
}
code[31]=$(declare -f fun_31)


function fun_32 {
	oldIFS=$IFS
	IFS=$'\n'
	
	if [ -s /etc/syslog.conf ]
	then
        	for line in $(cat /etc/syslog.conf)
        	do
                	if test "`echo $line | grep -v ^[[:space:]]*# | awk '{print $1}' | gawk '/^[[:space:]]*(cron|\*\.)/{print $0}'`"
                	then
                        	cron=yes
                        	item=`echo $line | awk '{print $1}'`
                        	dir=`echo $line | awk '{print $2}'`
                        	#echo $item=$dir
                	fi
        	done
	else
        	cron=no
	fi
	
	if [ -s /etc/rsyslog.conf ]
	then
        	for line in $(cat /etc/rsyslog.conf)
        	do
                	if test "`echo $line | grep -v ^[[:space:]]*# | awk '{print $1}' | awk '/^[[:space:]]*(cron|\.\*)/'`"
                	then
                        	cron=yes
                        	item=`echo $line | awk '{print $1}'`
                        	dir=`echo $line | awk '{print $2}'`
                        	#echo $item=$dir
                	fi
        	done
	else
        	cron=no
	fi

	echo $cron

	IFS=$oldIFS

	unset oldIFS conf cron
}
code[32]=$(declare -f fun_32)


function fun_33 {
	if [[ -f /var/log/messages ]];then
		mess_chmod=`ls -l /var/log/messages | awk '{print $1}'`
    		six_chmod=`echo ${mess_chmod:5:1}`
    		if [ $six_chmod = - ]
    		then
        		echo yes
		else
			#lsattr $search
			echo no
		fi
	else
		echo no
	fi
    	unset mess_chmod six_chmod
}
code[33]=$(declare -f fun_33)


function fun_34 {
        flag=0
        for f in /proc/sys/net/ipv4/conf/*/accept_source_route
        do
                if [ "$f" == "/proc/sys/net/ipv4/conf/lo/accept_source_route" ]
                then
                        continue;
                fi

                result=`cat $f`
                if [ $result -ne 0 ] 
                then
                        let 'flag+=1'
                fi
        done
	
	kernel_p1="net.ipv4.conf.all.accept_source_route"
        kernel_p2="net.ipv4.conf.default.accept_source_route"
       	if [ $flag -eq 0 ];then
	       	if [ `sysctl -n $kernel_p1 | grep -w '0' 2>/dev/null 1>/dev/null; echo $?` -gt 0 ];then
                       let 'flag+=1'
               fi
               if [ `sysctl -n $kernel_p2 | grep -w '0' 2>/dev/null 1>/dev/null; echo $?` -gt 0 ];then
                       let 'flag+=1'
               fi
       fi


        if [ $flag -gt 0 ]
        then
                echo no
        else
                echo yes
        fi
}
code[34]=$(declare -f fun_34)


function fun_35 {
        num1=`cat /etc/host.conf | grep '^[[:space:]]*order[[:space:]]*hosts,bind[[:space:]]*'`
        num2=`cat /etc/host.conf | grep '^[[:space:]]*multi[[:space:]]*on[[:space:]]*'`

        #echo $num1  $num2  $num3
		if [[ -n "$num1" ]]&&[[ -n "$num2" ]]
        then
                echo yes
        else
                echo no
        fi
}
code[35]=$(declare -f fun_35)


function fun_36 {
	sys_attack=`cat /proc/sys/net/ipv4/tcp_syncookies`
	if [ $sys_attack -eq 1 ]
	then
		echo yes
	else
		echo no
	fi
}
code[36]=$(declare -f fun_36)


function fun_37 {
	HISTSIZE=`cat /etc/profile | grep HISTSIZE | head -1 | awk -F[=] '{print $2}'`
	#echo HISTSIZE=$HISTSIZE
	if [[ $HISTSIZE -le 5 ]]
	then
		echo yes
	else
		echo no
	fi
}
code[37]=$(declare -f fun_37)


function fun_38 {
	source ~/.bashrc
	alia_ls=`alias | grep -E 'ls='`
	alia_rm=`alias | grep -E 'rm='`
	#echo "$alia"
	if [[ -n "$alia_ls" ]] && [[ -n "$alia_rm" ]]
	then
		echo yes
	else
		echo no
	fi
}
code[38]=$(declare -f fun_38)



function fun_39 {
	if [[ $os_name = "kylin" ]]||[[ $os_name = "linx" ]]
	then
		#echo $os_name
		echo yes
	else
		echo no
	fi
}
code[39]=$(declare -f fun_39)


function fun_40 {
	email=`ps -ef | grep E-Mail | grep -v grep`
	web=`ps -ef | grep Web | grep -v grep`
	ftp=`ps -ef | grep FTP | grep -v grep`
	telnet=`ps -ef | grep telnet | grep -v grep`
	rlogin=`ps -ef | grep rlogin | grep -v grep`
	smb=`ps -ef | grep SMB | grep -v grep`
	
	if [[ -n "$email" ]] || [[ -n "$web" ]] || [[ -n "$ftp" ]] || [[ -n "$telnet" ]] || [[ -n "$rlogin" ]] || [[ -n $smb ]]
	then
		echo no
	else
		echo yes
	fi
	unset service email web ftp telnet rlogin
}
code[40]=$(declare -f fun_40)


function fun_41 {
	root_logins=0
	root_logins=`who | awk '{print $1}' | sed -n '/^root$/p' | wc -l`
	#echo root_logins=$root_logins
	if [ $root_logins -eq 0 ]
	then
		echo yes
	else
		echo no
	fi
}
code[41]=$(declare -f fun_41)


function fun_42 {
	count=0
	item=`netstat -nultp | grep -E ^'tcp|udp'`

	oldIFS=$IFS
	IFS=$'\n'

	for line in $item
	do
        	proto=`echo "$line"|awk '{print $1}'`
        	address=`echo "$line"|awk '{print $4}'`
        	socket=`echo "$address"|awk -F ':' '{print $2}'`
			count=$(expr $count + 1)
        	#echo $proto.$socket
	done

	# if [ $count -gt 15 ]
	# then
	# 	echo no
	# else
	# 	echo yes
	# fi
	echo yes
	IFS=$oldIFS
	unset oldIFS proto address socket count
}
code[42]=$(declare -f fun_42)



function fun_43 {
	file="/lib/modules/`uname -r`/kernel/drivers/usb/storage/usb-storage.ko"
	if [ -e $file ]
	then
		echo no
	else
		echo yes
	fi
}
code[43]=$(declare -f fun_43)


function fun_44 {
	if [[ $os_name = "linx" ]]
	then
		string1=`lsmod | grep lsm_linx`
		string2=`lsmod | grep linx_sec`
		if [[ -n $string1 ]] || [[ -n $string2 ]]
		then
			echo yes
		else
			echo yes
		fi
	elif [[ $os_name = "kylin" ]]
	then
		rbapol > /dev/null 2>&1 &
		if [[ $? -eq 0 ]]
		then
			echo yes
		else
			echo yes
		fi
	else
		echo yes
	fi
}
code[44]=$(declare -f fun_44)


function fun_45 {
	service=`ps aux | grep -w auditd | grep -v grep`
	if test "$service"
	then
        audit=1
        if [ -s /var/log/audit/audit.log ]
        then
            #ls_show=`ls -l /var/log/audit/audit.log`
            #lsattr_show=`lsattr /var/log/audit/audit.log`
			echo yes
		else
			echo yes
        fi
	else
        echo no
	fi

	#echo audit=$audit
	#echo ls=$ls_show
	#echo lasttr=$lsattr_show

	unset service audit ls_show lsattr_show
}
code[45]=$(declare -f fun_45)


function fun_46 {
	file=`find / -name SCADA.p12 -o -name SCADA.cer 2> /dev/null`

	if test "$file"
	then
        	scada=1
			echo yes
	else
        	scada=0
			echo no
	fi

	#echo scada=$scada

	unset file scada
}
code[46]=$(declare -f fun_46)


function fun_47 {
	file=`find / -name AGC.p12 -o -name SCADA.cer 2> /dev/null`
	if test "$file"
	then
        	AGC=1
			echo yes
	else
        	AGC=0
			echo no
	fi

	#echo AGC=$AGC

	unset file AGC
}
code[47]=$(declare -f fun_47)


function fun_48 {
	count=0
    	while read line
    	do
        	uid=`echo $line | awk -F ':' '{print $3}'`
        	if [ $uid -eq 0 ]
        	then
            		((count=$count-1))
        	fi
    	done < /etc/passwd
    	if [ $count -le 1 ]
    	then    
       		echo yes
    	else    
        	echo no
    	fi
}
code[48]=$(declare -f fun_48)


function fun_49 {
        oldIFS=$IFS
        IFS=$'\n'

        for line in $(cat /etc/security/limits.conf)
        do
                if [ `echo $line | grep -v "[[:space:]]*#" | awk '{print $3}' | grep -x "stack"` ]; then
                        stack=`echo $line | awk '{print $4}'`
                elif [ `echo $line | grep -v "[[:space:]]*#" | awk '{print $3}' | grep -x "rss"` ]; then
                        rss=`echo $line | awk '{print $4}'`
                fi
        done

        if [ -z "$stack" ] || [ -z "$rss" ]; then
                echo no
        else
                echo yes
        fi
        IFS=$oldIFS
        unset oldIFS line
}
code[49]=$(declare -f fun_49)


function fun_50 {
	kernel=`uname`
	kernel_version=`uname -v`
	echo yes
}
code[50]=$(declare -f fun_50)

function fun_51 {
	core_hard=`cat /etc/security/limits.conf | grep -v "[[:space:]]*#" | grep "[[:space:]]*[[:space:]]*hard[[:space:]]*core[[:space:]]*[[:digit:]]" | awk '{print $4}'`
	core_soft=`cat /etc/security/limits.conf | grep -v "[[:space:]]*#" | grep "[[:space:]]*[[:space:]]*soft[[:space:]]*core[[:space:]]*[[:digit:]]" | awk '{print $4}'`
	if [[ $core_hard -eq 0 ]] && [[ $core_soft -eq 0 ]]
	then 
		echo yes
	else
		echo no
	fi
}
code[51]=$(declare -f fun_51)



ipaddr=$(echo $1 | awk -F '.' '$1<255 && $1>=0 && $2<255 && $2>=0 && $3<255 && $3>=0 && $4<255 && $4>=0 {print 1}') 

if [ $ipaddr -ne 1 ]
then
    echo Usage:ip address wrong
    exit -1
fi

if [ $# -gt 3 ]
then
        echo Usage:too many args,ignore the fourth !
elif [ $# -lt 3 ]
then
	echo Usage:lack of args!
	exit -1
fi

oldIFS=$IFS
IFS=,
count=0

for (( i=0; i <= 51; i++ ))
do
        arry[$i]=0
done

for arg in $2
do
        if test "`echo $arg | grep -x -E "[0-9]|[1-4][0-9]|5[0-1]"`"
        then
                arry[$arg]=$[${arry[$arg]} + 1]
                if [[ ${arry[$arg]} -gt 1 ]]
                then
                        echo "arg "$arg" repeated emergence,only handle once"
                else
                        count=$[$count + 1]
                fi
        else
                echo "arg "$arg" \"> 51\" or \"< 0\" or \"not a number\",ignore it"
        fi
done

IFS=$oldIFS


date=`date "+%Y-%m-%d %H:%M:%S"`
coding=`echo $LANG`
coding_value="UTF-8"

if test "`echo $coding | grep GB`"
then
	coding_value="GBK"
fi

xmlfile=$3".xml"
#echo $xmlfile

if [ -e /tmp/$xmlfile ]
then
	rm -f /tmp/$xmlfile
#	echo "$xmlfile exists,remove it !"
fi

touch /tmp/$xmlfile

echo "<?xml version='1.0' encoding='$coding_value'?>" > /tmp/$xmlfile
echo "<result>" >> /tmp/$xmlfile
echo "<osName><![CDATA["$os_name"]]></osName>" >> /tmp/$xmlfile
echo "<version><![CDATA["$os_version"]]></version>" >> /tmp/$xmlfile
echo "<ip><![CDATA["$1"]]></ip>" >> /tmp/$xmlfile
echo "<type><![CDATA[/server/"$os_name"]]></type>" >> /tmp/$xmlfile
echo "<startTime><![CDATA["$date"]]></startTime>" >> /tmp/$xmlfile
echo "<pId><![CDATA[$$]]></pId>" >> /tmp/$xmlfile

echo -e "\t<scripts>" >> /tmp/$xmlfile

if [[ ${arry[0]} -gt 0 ]]&&[[ $count -eq 1 ]]
then
	for (( i=1; i <= 51; i++ ))
        do
                        echo -e "\t\t<script>" >> /tmp/$xmlfile
                        echo -e "\t\t\t<id>"$i"</id>" >> /tmp/$xmlfile
                        echo -e "\t\t\t<code><![CDATA[${code[$i]}]]></code>" >> /tmp/$xmlfile
                        #echo "$i"
                        value=$(fun_$i)
                        echo -e "\t\t\t<value><![CDATA[$value]]></value>" >> /tmp/$xmlfile
                        echo -e "\t\t</script>" >> /tmp/$xmlfile
        done
elif [[ ${arry[0]} -gt 0 ]]&&[[ $count -gt 1 ]]
then
        echo arg "0" means test all the items,ignore other args
        for (( i=1; i <= 51; i++ ))
        do
                        echo -e "\t\t<script>" >> /tmp/$xmlfile
                        echo -e "\t\t\t<id>"$i"</id>" >> /tmp/$xmlfile
                        echo -e "\t\t\t<code><![CDATA[${code[$i]}]]></code>" >> /tmp/$xmlfile
                        #echo "$i"
                        value=$(fun_$i)
                        echo -e "\t\t\t<value><![CDATA[$value]]></value>" >> /tmp/$xmlfile
                        echo -e "\t\t</script>" >> /tmp/$xmlfile
        done
else
        for (( i=1; i <= 51; i++ ))
        do
                if [ ${arry[$i]} -gt 0 ]
                then
                        echo -e "\t\t<script>" >> /tmp/$xmlfile
                        echo -e "\t\t\t<id>"$i"</id>" >> /tmp/$xmlfile
                        echo -e "\t\t\t<code><![CDATA[${code[$i]}]]></code>" >> /tmp/$xmlfile
			#echo "$i"
                        value=$(fun_$i)
                        echo -e "\t\t\t<value><![CDATA[$value]]></value>" >> /tmp/$xmlfile
			echo -e "\t\t</script>" >> /tmp/$xmlfile

                fi
        done
fi

echo -e "\t</scripts>" >> /tmp/$xmlfile
enddate=`date "+%Y-%m-%d %H:%M:%S"`
echo "<endTime><![CDATA["$enddate"]]></endTime>" >> /tmp/$xmlfile
echo "</result>" >> /tmp/$xmlfile
#echo -e "write  result to xml file\nexecute end!\n"
